/*************************************************************************
*
* ADOBE CONFIDENTIAL
* ___________________
*
*  Copyright 2004 Adobe Systems Incorporated
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Adobe Systems Incorporated and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Adobe Systems Incorporated and its
* suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Adobe Systems Incorporated.
**************************************************************************/
/*
 * npunix.c
 *
 * Netscape Client Plugin API
 * - Wrapper function to interface with the Netscape Navigator
 *
 * dp Suresh <dp@netscape.com>
 *
 *----------------------------------------------------------------------
 * PLUGIN DEVELOPERS:
 *	YOU WILL NOT NEED TO EDIT THIS FILE.
 *----------------------------------------------------------------------
 */

#define XP_UNIX 1

#if DEBUG
#define OutputDebugString(x) { FILE * f1 = fopen("/tmp/nppdf", "a"); fprintf(f1, "%s\n", x); fclose (f1); }
#else
#define OutputDebugString(x)
#endif

#include "stdio.h"
#include "npapi.h"
#include "npupp.h"

#include "nsDefaultPlugin.h"
/*
 * Define PLUGIN_TRACE to have the wrapper functions print
 * messages to stderr whenever they are called.
 */

#ifdef PLUGIN_TRACE
#include <stdio.h>
#define PLUGINDEBUGSTR(msg)	fprintf(stderr, "%s\n", msg)
#else
#define PLUGINDEBUGSTR(msg)
#endif


/***********************************************************************
 *
 * Globals
 *
 ***********************************************************************/

static NPNetscapeFuncs   gNetscapeFuncs;	/* Netscape Function table */


/***********************************************************************
 *
 * Wrapper functions : plugin calling Netscape Navigator
 *
 * These functions let the plugin developer just call the APIs
 * as documented and defined in npapi.h, without needing to know
 * about the function table and call macros in npupp.h.
 *
 ***********************************************************************/

NPError
NPN_GetValue(NPP instance, NPNVariable variable, void *r_value)
{
	return CallNPN_GetValueProc(gNetscapeFuncs.getvalue, instance, variable, r_value);
}

void
NPN_Version(int* plugin_major, int* plugin_minor,
	     int* netscape_major, int* netscape_minor)
{
	*plugin_major = NP_VERSION_MAJOR;
	*plugin_minor = NP_VERSION_MINOR;

	/* Major version is in high byte */
	*netscape_major = gNetscapeFuncs.version >> 8;
	/* Minor version is in low byte */
	*netscape_minor = gNetscapeFuncs.version & 0xFF;
}

NPError
NPN_GetURL(NPP instance, const char* url, const char* window)
{
	return CallNPN_GetURLProc(gNetscapeFuncs.geturl, instance, url, window);
}

NPError
NPN_GetURLNotify(NPP instance, const char* url, const char* window, void * notifyData)
{
	return CallNPN_GetURLNotifyProc(gNetscapeFuncs.geturlnotify, instance,
			url, window, notifyData);
}

NPError
NPN_PostURL(NPP instance, const char* url, const char* window,
	     uint32 len, const char* buf, NPBool file)
{
	return CallNPN_PostURLProc(gNetscapeFuncs.posturl, instance,
					url, window, len, buf, file);
}

NPError
NPN_RequestRead(NPStream* stream, NPByteRange* rangeList)
{
	return CallNPN_RequestReadProc(gNetscapeFuncs.requestread,
					stream, rangeList);
}

NPError
NPN_NewStream(NPP instance, NPMIMEType type, const char *window,
	      NPStream** stream_ptr)
{
	return CallNPN_NewStreamProc(gNetscapeFuncs.newstream, instance,
					type, window, stream_ptr);
}

int32
NPN_Write(NPP instance, NPStream* stream, int32 len, void* buffer)
{
	return CallNPN_WriteProc(gNetscapeFuncs.write, instance,
					stream, len, buffer);
}

NPError
NPN_DestroyStream(NPP instance, NPStream* stream, NPError reason)
{
	return CallNPN_DestroyStreamProc(gNetscapeFuncs.destroystream,
						instance, stream, reason);
}

void
NPN_Status(NPP instance, const char* message)
{
	CallNPN_StatusProc(gNetscapeFuncs.status, instance, message);
}

const char*
NPN_UserAgent(NPP instance)
{
	return CallNPN_UserAgentProc(gNetscapeFuncs.uagent, instance);
}

void*
NPN_MemAlloc(uint32 size)
{
	return CallNPN_MemAllocProc(gNetscapeFuncs.memalloc, size);
}

void NPN_MemFree(void* ptr)
{
	CallNPN_MemFreeProc(gNetscapeFuncs.memfree, ptr);
}

uint32 NPN_MemFlush(uint32 size)
{
	return CallNPN_MemFlushProc(gNetscapeFuncs.memflush, size);
}

void NPN_ReloadPlugins(NPBool reloadPages)
{
	CallNPN_ReloadPluginsProc(gNetscapeFuncs.reloadplugins, reloadPages);
}

//extern "C" NPObject *CreateNPObject(NPP npp, NPClass *npclass);

NPObject *NPN_CreateObject(NPP instance, NPClass *npclass)
{
	return gNetscapeFuncs.createobject(instance, npclass);
}

NPUTF8 *NPN_UTF8FromIdentifier(NPIdentifier name)
{
	return gNetscapeFuncs.utf8fromidentifier(name);
}

NPObject *NPN_RetainObject(NPObject * obj)
{
	return gNetscapeFuncs.retainobject(obj);
}

bool NPN_Evaluate(NPP npp, NPObject *npobj, NPString *script, NPVariant *result)
{
	return gNetscapeFuncs.evaluate(npp, npobj, script, result);
}

bool NPN_Invoke(NPP npp, NPObject *npobj, NPIdentifier methodName, const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	return gNetscapeFuncs.invoke(npp, npobj, methodName, args, argCount, result);
}

void NPN_ReleaseVariantValue(NPVariant *result)
{
	gNetscapeFuncs.releasevariantvalue(result);
}

bool NPN_GetProperty(NPP npp, NPObject *npobj, NPIdentifier id, NPVariant *result)
{
	return gNetscapeFuncs.getproperty(npp, npobj, id, result);
}

NPIdentifier NPN_GetStringIdentifier(const NPUTF8 *str)
{
	return gNetscapeFuncs.getstringidentifier(str);
}

bool NPN_HasProperty(NPP npp, NPObject *npobj, NPIdentifier propertyName)
{
	return gNetscapeFuncs.hasproperty(npp, npobj, propertyName);
}

void NPN_ReleaseObject(NPObject *npobj)
{
	gNetscapeFuncs.releaseobject(npobj);
}

bool NPN_InvokeDefault(NPP npp, NPObject *npobj,  const NPVariant *args, uint32_t argCount, NPVariant *result)
{
	return gNetscapeFuncs.invokeDefault(npp, npobj, args, argCount, result);
}

bool NPN_SetProperty(NPP npp, NPObject *npobj, NPIdentifier propertyName, const NPVariant *value)
{
	return gNetscapeFuncs.setproperty(npp, npobj, propertyName, value);
}

/***********************************************************************
 *
 * Wrapper functions : Netscape Navigator -> plugin
 *
 * These functions let the plugin developer just create the APIs
 * as documented and defined in npapi.h, without needing to 
 * install those functions in the function table or worry about
 * setting up globals for 68K plugins.
 *
 ***********************************************************************/

NPError
Private_New(NPMIMEType pluginType, NPP instance, uint16 mode,
		int16 argc, char* argn[], char* argv[], NPSavedData* saved)
{
	NPError ret;
	PLUGINDEBUGSTR("New");
	ret = NPP_New(pluginType, instance, mode, argc, argn, argv, saved);
	return ret;	
}

NPError
Private_Destroy(NPP instance, NPSavedData** save)
{
	PLUGINDEBUGSTR("Destroy");
	return NPP_Destroy(instance, save);
}

NPError
Private_SetWindow(NPP instance, NPWindow* window)
{
	NPError err;
	PLUGINDEBUGSTR("SetWindow");
	err = NPP_SetWindow(instance, window);
	return err;
}

NPError
Private_NewStream(NPP instance, NPMIMEType type, NPStream* stream,
			NPBool seekable, uint16* stype)
{
	NPError err;
	PLUGINDEBUGSTR("NewStream");
	err = NPP_NewStream(instance, type, stream, seekable, stype);
	return err;
}

int32
Private_WriteReady(NPP instance, NPStream* stream)
{
	unsigned int result;
	PLUGINDEBUGSTR("WriteReady");
	result = NPP_WriteReady(instance, stream);
	return result;
}

int32
Private_Write(NPP instance, NPStream* stream, int32 offset, int32 len,
		void* buffer)
{
	unsigned int result;
	PLUGINDEBUGSTR("Write");
	result = NPP_Write(instance, stream, offset, len, buffer);
	return result;
}

void
Private_StreamAsFile(NPP instance, NPStream* stream, const char* fname)
{
	PLUGINDEBUGSTR("StreamAsFile");
	NPP_StreamAsFile(instance, stream, fname);
}


NPError
Private_DestroyStream(NPP instance, NPStream* stream, NPError reason)
{
	NPError err;
	PLUGINDEBUGSTR("DestroyStream");
	err = NPP_DestroyStream(instance, stream, reason);
	return err;
}


void
Private_Print(NPP instance, NPPrint* platformPrint)
{
	PLUGINDEBUGSTR("Print");
	NPP_Print(instance, platformPrint);
}


/*********************************************************************** 
 *
 * These functions are located automagically by netscape.
 *
 ***********************************************************************/

/*
 * NP_GetMIMEDescription
 *	- Netscape needs to know about this symbol
 *	- Netscape uses the return value to identify when an object instance
 *	  of this plugin should be created.
 */
char *
NP_GetMIMEDescription(void)
{
	return NPP_GetMIMEDescription();
}


/*
 * NP_GetValue
 *     - Netscape needs to know about this symbol.
 *     - It calls this function to query the plugin name and description.
 */
NPError NP_GetValue(void *instance, NPPVariable variable, void *value)
{
	OutputDebugString("NP_GETVALUE");
	return NPP_GetValue(instance, variable, value);
}


/*
 * NP_Initialize
 *	- Netscape needs to know about this symbol.
 *	- It calls this function after looking up its symbol before it
 *	  is about to create the first ever object of this kind.
 *
 * PARAMETERS
 *    nsTable	- The netscape function table. If developers just use these
 *		  wrappers, they dont need to worry about all these function
 *		  tables.
 * RETURN
 *    pluginFuncs
 *		- This functions needs to fill the plugin function table
 *		  pluginFuncs and return it. Netscape Navigator plugin
 *		  library will use this function table to call the plugin.
 *
 */
NPError
NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
{
	NPError err = NPERR_NO_ERROR;
	OutputDebugString("NP_Initialize");

	PLUGINDEBUGSTR("NP_Initialize");
	
	/* validate input parameters */

	if ((nsTable == NULL) || (pluginFuncs == NULL))
		err = NPERR_INVALID_FUNCTABLE_ERROR;
	
	/*
	 * Check the major version passed in Netscape's function table.
	 * We won't load if the major version is newer than what we expect.
	 * Also check that the function tables passed in are big enough for
	 * all the functions we need (they could be bigger, if Netscape added
	 * new APIs, but that's OK with us -- we'll just ignore them).
	 *
	 */

	if (err == NPERR_NO_ERROR)
	{
		if ((nsTable->version >> 8) > NP_VERSION_MAJOR)
		{
			err = NPERR_INCOMPATIBLE_VERSION_ERROR;
			OutputDebugString("NP_Initialize:version  mismatch");
		}

	/*
	 * We won't be performing these checks anymore, in order to support for
	 * all the versions of the browser. This could result in limited/irratic
	 * behaviour in older versions, but that would happen only when trying
	 * to use special features like javascript communication in embeded windows.
	 * Since this would not be as wide spread as simple viewing of PDFs in 
	 * the browser, we do not want the main functionality to be hindered by this.
	 */

#if 0
		if (nsTable->size < sizeof(NPNetscapeFuncs))
		{
			err = NPERR_INVALID_FUNCTABLE_ERROR;
			OutputDebugString("NP_Initialize NPNetscapeFuncs size mismatch");
		}
#endif
	}
		
	
	if (err == NPERR_NO_ERROR) {
		/*
		 * Copy all the fields of Netscape function table into our
		 * copy so we can call back into Netscape later.  Note that
		 * we need to copy the fields one by one, rather than assigning
		 * the whole structure, because the Netscape function table
		 * could actually be bigger than what we expect.
		 */
		gNetscapeFuncs.version       = nsTable->version;
		gNetscapeFuncs.size          = nsTable->size;
		gNetscapeFuncs.posturl       = nsTable->posturl;
		gNetscapeFuncs.geturl        = nsTable->geturl;
		gNetscapeFuncs.requestread   = nsTable->requestread;
		gNetscapeFuncs.newstream     = nsTable->newstream;
		gNetscapeFuncs.write         = nsTable->write;
		gNetscapeFuncs.destroystream = nsTable->destroystream;
		gNetscapeFuncs.status        = nsTable->status;
		gNetscapeFuncs.uagent        = nsTable->uagent;
		gNetscapeFuncs.memalloc      = nsTable->memalloc;
		gNetscapeFuncs.memfree       = nsTable->memfree;
		gNetscapeFuncs.memflush      = nsTable->memflush;
		gNetscapeFuncs.reloadplugins = nsTable->reloadplugins;
		gNetscapeFuncs.getvalue      = nsTable->getvalue;
		gNetscapeFuncs.setvalue      = nsTable->setvalue;
		
		gNetscapeFuncs.createobject = nsTable->createobject;
		gNetscapeFuncs.utf8fromidentifier = nsTable->utf8fromidentifier;
		gNetscapeFuncs.evaluate     = nsTable->evaluate;
		gNetscapeFuncs.retainobject = nsTable->retainobject;
		gNetscapeFuncs.invoke       = nsTable->invoke;
		gNetscapeFuncs.releasevariantvalue = nsTable->releasevariantvalue;
		gNetscapeFuncs.getproperty  = nsTable->getproperty;
		gNetscapeFuncs.getstringidentifier = nsTable->getstringidentifier;
		
		gNetscapeFuncs.hasproperty = nsTable->hasproperty;
		gNetscapeFuncs.releaseobject = nsTable->releaseobject;
		gNetscapeFuncs.invokeDefault = nsTable->invokeDefault;
		gNetscapeFuncs.setproperty = nsTable->setproperty;

		/*
		 * Set up the plugin function table that Netscape will use to
		 * call us.  Netscape needs to know about our version and size
		 * and have a UniversalProcPointer for every function we
		 * implement.
		 */
		pluginFuncs->version    =
				(NP_VERSION_MAJOR << 8) + 14; //NP_VERSION_MINOR
		pluginFuncs->size       = sizeof(NPPluginFuncs);
		pluginFuncs->newp       = NewNPP_NewProc(Private_New);
		pluginFuncs->destroy    = NewNPP_DestroyProc(Private_Destroy);
		pluginFuncs->setwindow  =
				NewNPP_SetWindowProc(Private_SetWindow);
		pluginFuncs->newstream  =
				NewNPP_NewStreamProc(Private_NewStream);
		pluginFuncs->destroystream =
				NewNPP_DestroyStreamProc(Private_DestroyStream);
		pluginFuncs->asfile     =
				NewNPP_StreamAsFileProc(Private_StreamAsFile);
		pluginFuncs->writeready =
				NewNPP_WriteReadyProc(Private_WriteReady);
		pluginFuncs->write      = NewNPP_WriteProc(Private_Write);
		pluginFuncs->print      = NewNPP_PrintProc(Private_Print);

		pluginFuncs->getvalue = (NPP_GetValueUPP)NPP_GetValue;
		pluginFuncs->setvalue = (NPP_SetValueUPP)NPP_SetValue;

		pluginFuncs->event      = NULL;

		err = NPP_Initialize();
	}
	
	return err;
}

/*
 * NP_Shutdown [optional]
 *	- Netscape needs to know about this symbol.
 *	- It calls this function after looking up its symbol after
 *	  the last object of this kind has been destroyed.
 *
 */
NPError
NP_Shutdown(void)
{
 	PLUGINDEBUGSTR("NP_Shutdown");
	NPP_Shutdown();
	return NPERR_NO_ERROR;
}
